@@ -1,6 +1,6 @@
 ;; geiser-racket.el -- geiser support for Racket scheme
 
-;; Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Jose Antonio Ortega Ruiz
+;; Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2015 Jose Antonio Ortega Ruiz
 
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the Modified BSD License. You should
@@ -20,6 +20,8 @@
 (require 'geiser-base)
 (require 'geiser)
 
+(require 'compile)
+
 (eval-when-compile (require 'cl))
 
 
@@ -178,7 +180,7 @@ using start-geiser, a procedure in the geiser/server module."
         (t nil)))
 
 (defun geiser-racket--symbol-begin (module)
-  (save-excursion (skip-syntax-backward "^-()>") (point)))
+  (save-excursion (skip-syntax-backward "^'-()>") (point)))
 
 (defun geiser-racket--import-command (module)
   (and (stringp module)
@@ -263,7 +265,7 @@ using start-geiser, a procedure in the geiser/server module."
 
 ;;; Keywords and syntax
 
-(setq geiser-racket-font-lock-forms
+(defvar geiser-racket-font-lock-forms
   '(("^#lang\\>" . 0)
     ("\\[\\(else\\)\\>" . 1)
     ("(\\(define/match\\)\\W+[[(]?\\(\\w+\\)+\\b"
@@ -272,10 +274,7 @@ using start-geiser, a procedure in the geiser/server module."
 
 (defun geiser-racket--keywords ()
   (append geiser-racket-font-lock-forms
-          (when geiser-racket-extra-keywords
-            `((,(format "[[(]%s\\>"
-                        (regexp-opt geiser-racket-extra-keywords 1))
-               . 1)))))
+          (geiser-syntax--simple-keywords geiser-racket-extra-keywords)))
 
 (geiser-syntax--scheme-indent
  (begin0 1)
@@ -366,6 +365,8 @@ using start-geiser, a procedure in the geiser/server module."
 (defun geiser-racket--version (binary)
   (shell-command-to-string (format "%s  -e '(display (version))'" binary)))
 
+(defvar geiser-racket--image-cache-dir nil)
+
 (defun geiser-racket--startup (remote)
   (set (make-local-variable 'compilation-error-regexp-alist)
        `(("^ *\\([^:(\t\n]+\\):\\([0-9]+\\):\\([0-9]+\\):" 1 2 3)))
@@ -373,9 +374,12 @@ using start-geiser, a procedure in the geiser/server module."
   (if geiser-image-cache-dir
       (geiser-eval--send/wait
        `(:eval (image-cache ,geiser-image-cache-dir) geiser/user))
-    (setq geiser-image-cache-dir
+    (setq geiser-racket--image-cache-dir
           (geiser-eval--send/result '(:eval (image-cache) geiser/user)))))
 
+(defun geiser-racket--image-cache-dir ()
+  (or geiser-image-cache-dir geiser-racket--image-cache-dir))
+
 
 ;;; Additional commands
 
@@ -434,6 +438,7 @@ Use a prefix to be asked for a submodule name."
   (external-help geiser-racket--external-help)
   (check-buffer geiser-racket--guess)
   (keywords geiser-racket--keywords)
+  (image-cache-dir geiser-racket--image-cache-dir)
   (case-sensitive geiser-racket-case-sensitive-p)
   (binding-forms geiser-racket--binding-forms)
   (binding-forms* geiser-racket--binding-forms*))
