@@ -1,6 +1,6 @@
 ;;; geiser-syntax.el -- utilities for parsing scheme syntax
 
-;; Copyright (C) 2009, 2010, 2011, 2012, 2013 Jose Antonio Ortega Ruiz
+;; Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014, 2015 Jose Antonio Ortega Ruiz
 
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the Modified BSD License. You should
@@ -28,34 +28,104 @@
                     pairs)))
 
 (geiser-syntax--scheme-indent
+ (and-let* 1)
  (case-lambda 0)
  (catch defun)
  (class defun)
  (dynamic-wind 0)
+ (guard 1)
  (let*-values 1)
- (letrec* 1)
- (letrec-values 1)
  (let-values 1)
  (let/ec 1)
- (match defun)
+ (letrec* 1)
+ (match 1)
+ (match-lambda 0)
+ (match-lambda* 0)
+ (match-let 1)
+ (match-let* 1)
+ (match-letrec 1)
  (opt-lambda 1)
  (parameterize 1)
  (parameterize* 1)
  (receive 2)
+ (require-extension 0)
  (syntax-case 2)
+ (test-approximate 1)
+ (test-assert 1)
+ (test-eq 1)
+ (test-equal 1)
+ (test-eqv 1)
+ (test-group-with-cleanup 1)
+ (test-runner-on-bad-count! 1)
+ (test-runner-on-bad-end-name! 1)
+ (test-runner-on-final! 1)
+ (test-runner-on-group-begin! 1)
+ (test-runner-on-group-end! 1)
+ (test-runner-on-test-begin! 1)
+ (test-runner-on-test-end! 1)
+ (test-with-runner 1)
  (unless 1)
  (when 1)
  (while 1)
- (with-error-to-port 1)
+ (with-exception-handler 1)
  (with-syntax 1))
 
 
 ;;; Extra syntax keywords
-(defconst geiser-syntax--keywords
-  '(("\\[\\(else\\)\\>" . 1)
-    ("(\\(parameterize\\)\\>" . 1)))
 
-(font-lock-add-keywords 'scheme-mode geiser-syntax--keywords)
+(defconst geiser-syntax--builtin-keywords
+  '("and-let*"
+    "cut"
+    "cute"
+    "define-condition-type"
+    "define-immutable-record-type"
+    "define-record-type"
+    "define-values"
+    "letrec*"
+    "match"
+    "match-lambda"
+    "match-lambda*"
+    "match-let"
+    "match-let*"
+    "match-letrec"
+    "parameterize"
+    "receive"
+    "require-extension"
+    "set!"
+    "syntax-case"
+    "test-approximate"
+    "test-assert"
+    "test-begin"
+    "test-end"
+    "test-eq"
+    "test-equal"
+    "test-eqv"
+    "test-error"
+    "test-group"
+    "test-group-with-cleanup"
+    "test-with-runner"
+    "unless"
+    "when"
+    "with-exception-handler"
+    "with-input-from-file"
+    "with-output-to-file"))
+
+(defun geiser-syntax--simple-keywords (keywords)
+  "Return `font-lock-keywords' to highlight scheme KEYWORDS.
+KEYWORDS should be a list of strings."
+  (when keywords
+    `((,(format "[[(]%s\\>" (regexp-opt keywords 1)) . 1))))
+
+(defun geiser-syntax--keywords ()
+  (append
+   (geiser-syntax--simple-keywords geiser-syntax--builtin-keywords)
+   `(("\\[\\(else\\)\\>" . 1)
+     (,(rx "(" (group "define-syntax-rule") eow (* space)
+           (? "(") (? (group (1+ word))))
+      (1 font-lock-keyword-face)
+      (2 font-lock-function-name-face nil t)))))
+
+(font-lock-add-keywords 'scheme-mode (geiser-syntax--keywords))
 
 (geiser-impl--define-caller geiser-syntax--impl-kws keywords ()
   "A variable (or thunk returning a value) giving additional,
@@ -219,22 +289,6 @@ implementation-specific entries for font-lock-keywords.")
                    sep
                    (geiser-syntax--mapconcat fun (cdr lst) sep)))))
 
-(defun geiser-syntax--display (a)
-  (cond ((null a) "()")
-        ((eq a :t) "#t")
-        ((eq a :f) "#f")
-        ((geiser-syntax--keywordp a) (format "#%s" a))
-        ((symbolp a) (format "%s" a))
-        ((equal a "...") "...")
-        ((stringp a) (format "%S" a))
-        ((and (listp a) (symbolp (car a))
-              (equal (symbol-name (car a)) "quote"))
-         (format "'%s" (geiser-syntax--display (cadr a))))
-        ((listp a)
-         (format "(%s)"
-                 (geiser-syntax--mapconcat 'geiser-syntax--display a " ")))
-        (t (format "%s" a))))
-
 
 ;;; Code parsing:
 
@@ -250,6 +304,10 @@ implementation-specific entries for font-lock-keywords.")
 (defsubst geiser-syntax--nesting-level ()
   (or (nth 0 (syntax-ppss)) 0))
 
+(defun geiser-syntax--pop-to-top ()
+  (ignore-errors
+    (while (> (geiser-syntax--nesting-level) 0) (backward-up-list))))
+
 (defsubst geiser-syntax--in-string-p ()
   (nth 3 (syntax-ppss)))
 
@@ -282,7 +340,7 @@ implementation-specific entries for font-lock-keywords.")
          (smth (or fst (not (looking-at-p "[\s \s)\s>\s<\n]"))))
          (path (and fst `((,fst 0)))))
     (save-excursion
-      (while (not (zerop (geiser-syntax--nesting-level)))
+      (while (> (or (geiser-syntax--nesting-level) 0) 0)
         (let ((boundary (point)))
           (geiser-syntax--skip-comment/string)
           (backward-up-list)
@@ -378,8 +436,7 @@ implementation-specific entries for font-lock-keywords.")
         (skip-syntax-forward "->")
         (let ((boundary (point))
               (nesting (geiser-syntax--nesting-level)))
-          (while (not (zerop (geiser-syntax--nesting-level)))
-            (backward-up-list))
+          (geiser-syntax--pop-to-top)
           (multiple-value-bind (form end)
               (geiser-syntax--form-after-point boundary)
             (delete sym
@@ -390,7 +447,23 @@ implementation-specific entries for font-lock-keywords.")
                                                 '()))))))))
 
 
-;;; Fontify strings as Scheme code:
+;;; Display and fontify strings as Scheme code:
+
+(defun geiser-syntax--display (a)
+  (cond ((null a) "()")
+        ((eq a :t) "#t")
+        ((eq a :f) "#f")
+        ((geiser-syntax--keywordp a) (format "#%s" a))
+        ((symbolp a) (format "%s" a))
+        ((equal a "...") "...")
+        ((stringp a) (format "%S" a))
+        ((and (listp a) (symbolp (car a))
+              (equal (symbol-name (car a)) "quote"))
+         (format "'%s" (geiser-syntax--display (cadr a))))
+        ((listp a)
+         (format "(%s)"
+                 (geiser-syntax--mapconcat 'geiser-syntax--display a " ")))
+        (t (format "%s" a))))
 
 (defun geiser-syntax--font-lock-buffer ()
   (let ((name " *geiser font lock*"))
@@ -408,7 +481,11 @@ implementation-specific entries for font-lock-keywords.")
     (set-buffer (geiser-syntax--font-lock-buffer))
     (erase-buffer)
     (insert str)
-    (let ((font-lock-verbose nil)) (font-lock-fontify-buffer))
+    (let ((font-lock-verbose nil))
+      (if (fboundp 'font-lock-ensure)
+          (font-lock-ensure)
+        (with-no-warnings
+          (font-lock-fontify-buffer))))
     (buffer-string)))
 
 
